@@ -27,7 +27,6 @@
 ;;; Code:
 
 (require 'cl-lib)
-(require 'haskell-compat)
 (require 'haskell-lexeme)
 (require 'font-lock)
 
@@ -93,7 +92,8 @@ be disabled at that position."
   '("case" "class" "data" "default" "deriving" "do"
     "else" "if" "import" "in" "infix" "infixl"
     "infixr" "instance" "let" "module" "mdo" "newtype" "of"
-    "rec" "pattern" "proc" "then" "type" "where" "_")
+    "rec" "pattern" "proc" "signature" "then" "type" "where" "_"
+    "anyclass" "stock" "via")
   "Identifiers treated as reserved keywords in Haskell."
   :group 'haskell-appearance
   :type '(repeat string))
@@ -108,6 +108,7 @@ This is the case if the \".\" is part of a \"forall <tvar> . <type>\"."
                             (line-beginning-position) t)
         (not (or
               (string= " " (string (char-after start)))
+              (null (char-before start))
               (string= " " (string (char-before start))))))))
 
 (defvar haskell-yesod-parse-routes-mode-keywords
@@ -133,6 +134,8 @@ This is the case if the \".\" is part of a \"forall <tvar> . <type>\"."
     ("r" . ess-mode)
     ("rChan" . ess-mode)
     ("sql" . sql-mode)
+    ("json" . json-mode)
+    ("aesonQQ" . json-mode)
     ("parseRoutes" . haskell-yesod-parse-routes-mode))
   "Mapping from quasi quoter token to fontification mode.
 
@@ -321,22 +324,27 @@ like ::, class, instance, data, newtype, type."
   "Private function used to select either type or constructor face
 on an uppercase identifier."
   (cl-case (haskell-lexeme-classify-by-first-char (char-after (match-beginning 1)))
-    (varid (when (member (match-string 0) haskell-font-lock-keywords)
-             ;; Note: keywords parse as keywords only when not qualified.
-             ;; GHC parses Control.let as a single but illegal lexeme.
-             (when (member (match-string 0) '("class" "instance" "type" "data" "newtype"))
-               (save-excursion
-                 (goto-char (match-end 0))
-                 (save-match-data
-                   (haskell-font-lock--forward-type
-                    (cond
-                     ((member (match-string 0) '("class" "instance"))
-                      '("|"))
-                     ((member (match-string 0) '("type"))
-                      ;; Need to support 'type instance'
-                      '("=" "instance")))))
-                 (add-text-properties (match-end 0) (point) '(font-lock-multiline t haskell-type t))))
-             'haskell-keyword-face))
+    (varid (let ((word (match-string-no-properties 0)))
+             (cond
+              ((member word haskell-font-lock-keywords)
+               ;; Note: keywords parse as keywords only when not qualified.
+               ;; GHC parses Control.let as a single but illegal lexeme.
+               (when (member word '("class" "instance" "type" "data" "newtype"))
+                 (save-excursion
+                   (goto-char (match-end 0))
+                   (save-match-data
+                     (haskell-font-lock--forward-type
+                      (cond
+                       ((member word '("class" "instance"))
+                        '("|"))
+                       ((member word '("type"))
+                        ;; Need to support 'type instance'
+                        '("=" "instance")))))
+                   (add-text-properties (match-end 0) (point) '(font-lock-multiline t haskell-type t))))
+               'haskell-keyword-face)
+              ((member word '("forall"))
+               (when (get-text-property (match-beginning 0) 'haskell-type)
+                 'haskell-keyword-face)))))
     (conid (if (get-text-property (match-beginning 0) 'haskell-type)
                'haskell-type-face
              'haskell-constructor-face))
@@ -696,7 +704,7 @@ on an uppercase identifier."
 (provide 'haskell-font-lock)
 
 ;; Local Variables:
-;; coding: utf-8-unix
+;; coding: utf-8
 ;; tab-width: 8
 ;; End:
 
