@@ -27,7 +27,7 @@
 (require 'haskell-modules)
 (require 'haskell-commands)
 (require 'haskell-session)
-(require 'haskell-utils)
+(require 'haskell-string)
 
 (defun haskell-process-look-config-changes (session)
   "Check whether a cabal configuration file has changed.
@@ -88,7 +88,14 @@ Restarts the SESSION's process if that is the case."
    ((haskell-process-consume process "\nBuilding \\(.+?\\)\\.\\.\\.")
     (let ((msg (format "Building: %s" (match-string 1 buffer))))
       (haskell-interactive-mode-echo (haskell-process-session process) msg)
-      (haskell-mode-message-line msg)))))
+      (haskell-mode-message-line msg)))
+   ((string-match "Collecting type info for [[:digit:]]+ module(s) \\.\\.\\."
+                  (haskell-process-response process)
+                  (haskell-process-response-cursor process))
+    (haskell-mode-message-line (match-string 0 buffer))
+    ;; Do not consume "Ok, modules loaded" that goes before
+    ;; "Collecting type info...", just exit.
+    nil)))
 
 (defun haskell-process-load-complete (session process buffer reload module-buffer &optional cont)
   "Handle the complete loading response. BUFFER is the string of
@@ -98,6 +105,22 @@ actual Emacs buffer of the module being loaded."
     (with-current-buffer (haskell-interactive-mode-splices-buffer session)
       (erase-buffer)))
   (let* ((ok (cond
+              ((haskell-process-consume
+                process
+                "Ok, \\(?:[0-9]+\\) modules? loaded\\.$")
+               t)
+               ((haskell-process-consume
+                process
+                "Ok, \\(?:[a-z]+\\) modules? loaded\\.$") ;; for ghc 8.4
+               t)
+              ((haskell-process-consume
+                process
+                "Failed, \\(?:[0-9]+\\) modules? loaded\\.$")
+               nil)
+              ((haskell-process-consume
+                process
+                "Failed, \\(?:[a-z]+\\) modules? loaded\\.$") ;; ghc 8.6.3 says so
+               nil)
               ((haskell-process-consume
                 process
                 "Ok, modules loaded: \\(.+\\)\\.$")
@@ -106,6 +129,10 @@ actual Emacs buffer of the module being loaded."
                 process
                 "Failed, modules loaded: \\(.+\\)\\.$")
                nil)
+	      ((haskell-process-consume
+                process
+                "Failed, no modules loaded\\.$") ;; for ghc 8.4
+               nil)
               (t
                (error (message "Unexpected response from haskell process.")))))
          (modules (haskell-process-extract-modules buffer))
@@ -173,6 +200,7 @@ list of modules where missed IDENT was found."
                  (not (string-match "\\([A-Z][A-Za-z]+\\) is deprecated" msg)))
             (string-match "Use \\([A-Z][A-Za-z]+\\) to permit this" msg)
             (string-match "Use \\([A-Z][A-Za-z]+\\) to allow" msg)
+            (string-match "Use \\([A-Z][A-Za-z]+\\) to enable" msg)
             (string-match
              "Use \\([A-Z][A-Za-z]+\\) if you want to disable this"
              msg)
@@ -193,7 +221,7 @@ list of modules where missed IDENT was found."
             file
             (match-string 2 msg)
             line)))
-        ((string-match "Warning: orphan instance: " msg)
+        ((string-match "[Ww]arning: orphan instance: " msg)
          (when haskell-process-suggest-no-warn-orphans
            (haskell-process-suggest-pragma
             session
@@ -252,6 +280,7 @@ list of modules where missed IDENT was found."
                              (cl-ecase (haskell-process-type)
                                ('ghci haskell-process-path-cabal)
                                ('cabal-repl haskell-process-path-cabal)
+                               ('cabal-new-repl haskell-process-path-cabal)
                                ('cabal-ghci haskell-process-path-cabal)
                                ('stack-ghci haskell-process-path-stack))
                              (cl-caddr state)))))
@@ -294,6 +323,7 @@ list of modules where missed IDENT was found."
                    :app-name (cl-ecase (haskell-process-type)
                                ('ghci haskell-process-path-cabal)
                                ('cabal-repl haskell-process-path-cabal)
+                               ('cabal-new-repl haskell-process-path-cabal)
                                ('cabal-ghci haskell-process-path-cabal)
                                ('stack-ghci haskell-process-path-stack))
                    :app-icon haskell-process-logo)))))))))))
@@ -314,7 +344,7 @@ list of modules where missed IDENT was found."
 (defun haskell-process-extract-modules (buffer)
   "Extract the modules from the process buffer."
   (let* ((modules-string (match-string 1 buffer))
-         (modules (split-string modules-string ", ")))
+         (modules (and modules-string (split-string modules-string ", "))))
     (cons modules modules-string)))
 
 ;;;###autoload
@@ -358,7 +388,7 @@ list of modules where missed IDENT was found."
   (with-current-buffer buffer
     (remove-overlays (point-min) (point-max) 'haskell-check t)))
 
-(defmacro with-overlay-properties (proplist ovl &rest body)
+(defmacro haskell-with-overlay-properties (proplist ovl &rest body)
   "Evaluate BODY with names in PROPLIST bound to the values of
 correspondingly-named overlay properties of OVL."
   (let ((ovlvar (cl-gensym "OVL-")))
@@ -366,26 +396,26 @@ correspondingly-named overlay properties of OVL."
             ,@(mapcar (lambda (p) `(,p (overlay-get ,ovlvar ',p))) proplist))
        ,@body)))
 
-(defun overlay-start> (o1 o2)
+(defun haskell-overlay-start> (o1 o2)
   (> (overlay-start o1) (overlay-start o2)))
-(defun overlay-start< (o1 o2)
+(defun haskell-overlay-start< (o1 o2)
   (< (overlay-start o1) (overlay-start o2)))
 
-(defun first-overlay-in-if (test beg end)
+(defun haskell-first-overlay-in-if (test beg end)
   (let ((ovls (cl-remove-if-not test (overlays-in beg end))))
-    (cl-first (sort (cl-copy-list ovls) 'overlay-start<))))
+    (cl-first (sort (cl-copy-list ovls) 'haskell-overlay-start<))))
 
-(defun last-overlay-in-if (test beg end)
+(defun haskell-last-overlay-in-if (test beg end)
   (let ((ovls (cl-remove-if-not test (overlays-in beg end))))
-    (cl-first (sort (cl-copy-list ovls) 'overlay-start>))))
+    (cl-first (sort (cl-copy-list ovls) 'haskell-overlay-start>))))
 
 (defun haskell-error-overlay-briefly (ovl)
-  (with-overlay-properties
+  (haskell-with-overlay-properties
    (haskell-msg haskell-msg-type) ovl
    (cond
     ((not (eq haskell-msg-type 'warning))
      haskell-msg)
-    ((string-prefix-p "Warning:\n    " haskell-msg)
+    ((string-prefix-p "[Ww]arning:\n    " haskell-msg)
      (cl-subseq haskell-msg 13))
     (t
      (error
@@ -402,7 +432,7 @@ correspondingly-named overlay properties of OVL."
 (defun haskell-goto-first-error ()
   (interactive)
   (haskell-goto-error-overlay
-   (first-overlay-in-if 'haskell-check-overlay-p
+   (haskell-first-overlay-in-if 'haskell-check-overlay-p
                         (buffer-end 0) (buffer-end 1))))
 
 (defun haskell-goto-prev-error ()
@@ -410,7 +440,7 @@ correspondingly-named overlay properties of OVL."
   (haskell-goto-error-overlay
    (let ((ovl-at
           (cl-first (haskell-check-filter-overlays (overlays-at (point))))))
-     (or (last-overlay-in-if 'haskell-check-overlay-p
+     (or (haskell-last-overlay-in-if 'haskell-check-overlay-p
                              (point-min)
                              (if ovl-at (overlay-start ovl-at) (point)))
          ovl-at))))
@@ -420,7 +450,7 @@ correspondingly-named overlay properties of OVL."
   (haskell-goto-error-overlay
    (let ((ovl-at
           (cl-first (haskell-check-filter-overlays (overlays-at (point))))))
-     (or (first-overlay-in-if
+     (or (haskell-first-overlay-in-if
           'haskell-check-overlay-p
           (if ovl-at (overlay-end ovl-at) (point)) (point-max))
          ovl-at))))
@@ -508,7 +538,7 @@ When MODULE-BUFFER is non-NIL, paint error overlays."
              (file (match-string 1 buffer))
              (location-raw (match-string 2 buffer))
              (error-msg (match-string 3 buffer))
-             (type (cond ((string-match "^Warning:" error-msg)  'warning)
+             (type (cond ((string-match "^[Ww]arning:" error-msg)  'warning)
                          ((string-match "^Splicing " error-msg) 'splice)
                          (t                                     'error)))
              (critical (not (eq type 'warning)))
@@ -522,7 +552,7 @@ When MODULE-BUFFER is non-NIL, paint error overlays."
                         (concat file ":" location-raw ": x")))
              (line (plist-get location :line))
              (col1 (plist-get location :col)))
-        (when module-buffer
+        (when (and module-buffer haskell-process-show-overlays)
           (haskell-check-paint-overlay
            module-buffer
            (string= (file-truename (buffer-file-name module-buffer))
