@@ -4,17 +4,13 @@
 (require 'haskell-font-lock)
 (require 'haskell-mode)
 
-;; Emacs 24.3 has sql-mode that runs without a product and therefore
-;; without font lock initially and needs to be extra enabled
-(add-hook 'sql-mode-hook (lambda () (sql-set-product 'ansi)))
-
 (ert-deftest haskell-syntactic-test-1 ()
   "Simple keywords fontified"
   (check-properties
-    '("module Test where")
-    '(("module" "w" haskell-keyword-face)
-      ("Test" "w" haskell-constructor-face)
-      ("where" "w" haskell-keyword-face))))
+   '("module Test where")
+   '(("module" "w" haskell-keyword-face)
+     ("Test" "w" haskell-constructor-face)
+     ("where" "w" haskell-keyword-face))))
 
 (ert-deftest haskell-syntactic-test-4 ()
   "Apostrophe as part of a contructor token."
@@ -662,6 +658,13 @@
     '(("foo" "w" haskell-definition-face)
       ("role" "w" nil))))
 
+(ert-deftest haskell-forall-in-type ()
+  (check-properties
+   '("forall = 23"
+     "zonk :: forall x . x -> x")
+   '(("forall" "w" haskell-definition-face)
+     ("forall" "w" haskell-keyword-face))))
+
 (ert-deftest haskell-unterminated-string-1 ()
   (check-properties
    '("foo = \"zonk"
